function fBestExper=APS11_Monte(funSet, funArray, budgetArray, Darray)
%{
 Used to launch APS11:
 Adaptive Population-based Simplex. Portfolio version
 Proposed by: Maurice Clerc and Mahamed G.H. Omran
 Version: 2017-06-10


 Inputs: funSet = 	1 real-world CEC 2011 benchmark
										2	CEC 2005 benchmark
										3 MyFunctions benchmark
										
 For each set, see get_fun_infos.m in the corresponding directory for details.																			
										
         funArray = array of function indices
 ---        
 Note: for Lennard-Jones (in MyFunctions), the function code is 1000+nb_of_atoms.
 For example, to run it with 10 atoms, the command is
 APS11_Monte(3,1010) with automatic limited budget (13400 in this case)
 or 
 APS11_Monte(3,1010,20000) with at most 20000 fitness evaluations 
 
 It means you can test for different numbers of atoms with a command like
 APS11_Monte(3,[1010 1020 1030])
 and the results are saved in different files					
 ---
					budgetArray = budget in terms of number of function evaluations (FEs)

					Darray = dimension of each problem. ONLY for CEC 2005


 ----  Example for all the CEC 2011 functions:
 funArray=[1:10 111:119,1110,12,13];
 Warning: apparently, there is a bug in the evaluation for 1110
					(constraint weigth set to zero). Therefore, it is identical to 118
	budgetArray=100*[60	134	24 24	134	134	109	64 275 84	268	360	60	88 94 154	289	240	240	240	124	114];
 APS11_Monte(1,funArray, budgetArray)

 If you want to use the default "limited budget" array, just
 APS11_Monte(1,funArray)

 For 150,000 FE on n functions of CEC2011
 APS11_Monte(1, [list of n functions], 150000*ones(1,n));
 
 ---- Example for 25 CEC 2005 functions (in dimension 10)
 APS11_Monte(2,[1:25],[],10*ones(1,25))
 
%}


global  initial_flag	% Needed for CEC 2005 benchmark
initial_flag = 0;
global diagonal zerosD onesD  % For the normalised search space
 
global xTot; xTot=[];% TEST, signature
global fTot; fTot=[]; % TEST, entropy
global savePos;% If true, saves all sampled positions and their values, for future tests
global signature; % If true, keeps all positions, and plots them  (only 2D)
savePos=false; 
signature=false; 

%------------------------------------------- Checks parameters
if exist('budgetArray','var')
  bA=length(budgetArray);
  fA=length(funArray);
  if bA>0 && bA ~= fA
    error('\nERROR: Number of budgets (%i) and functions (%i) are different',...
      bA,fA);
  end
end
%------------------------------------------------------Options, for test purpose
%fprintf('\n=========== APS11_Monte\n')

nbExperiment=100; % Number of experiments.
%{
			An "experiment" is the complete use of the "budget"
 			For fair comparisons, for a given problem, the best result
				of each experiment is recorded, and the _distribution_ of these results
				gives us a good idea of the performance of the method on this problem
				This distribution itself can be plotted, or at least summarised
				by (mean, std. dev, median).
				Note that for a correct estimation, nbExperiment should be
				at least 50. See the results in the files
			 Recap.txt, Table.txt, Exper.txt, Intermed.txt
				You may also apply later compareResults() for a comparison based on
				a user-defined quality function.
%}
restartRate=0.1; % 0.1. If positive, each time a stagnation seems probable
%							a specific treatment is applied
%
expAlpha=0.5;			% 0.5,  To modify expansion
contractGamma=-1 ; %1/3; % 1/3 To modify contraction. WARNING. Contraction may 
%											imply a bias towards the point contractAlpha*ones(1,D)/2);
%							-1   Negative value => automatic => reduces bias

strategArray=[1 2 3]; %[1 2 3]
%									 Portfolio of strategies
%									1 Expansion
%									2 Contraction
%									3 fmincon (Matlab Optimization toolbox, interior point method)
% TEST							4 minimize (see strategyApply for details)  
%
popSizeType = 0; %  0.   Uses a formula to define the population size (see popSize( ))
% 								if -N, then the size is N  (ex. -90)
keepRate =0.9 ; % 0.90

%------------------------
verbose=0; % 0 => almost no display
% 					 1 => more information (after each improvement, for example)
plotDistrib=0; % 1 => plots the distribution of the best results
nbBins=10; % Number of bins for the CDF plot

%---------------------------------------------Prepares files to save the results

stratString=num2str(strategArray);
stratString=strrep(stratString,' ','_');

results=createResults('APS11'); % Creates the directory for results, with date and time
  fprintf('\n Results will be saved in %s',results);

for funct = 1:length(funArray)
  fun=funArray(funct);
  
  FileName{funct} = [results, num2str(funSet), '[',num2str(fun),']'];

  fid(funct) = fopen(FileName{funct}, 'w');
  fprintf(fid(funct),' restartRate');
  fprintf(fid(funct),' expAlpha strategies popSizeType');
  fprintf(fid(funct),'  keepRate ');
  fprintf(fid(funct),'\n  %1.2f %1.2f %s %i  %f  \n', ...
    restartRate, expAlpha, stratString,  popSizeType, keepRate);
  
  FileNameT{funct}=[FileName{funct} 'Table.txt'];
  fidT(funct) = fopen(FileNameT{funct}, 'w');
  fprintf(fidT(funct),' restartRate');
  fprintf(fidT(funct),' expAlpha strategies popSizeType');
  fprintf(fidT(funct),' keepRate ');
  fprintf(fidT(funct),'\n   %1.2f %1.2f %s %i  %f  \n', ...
    restartRate, expAlpha, stratString,  popSizeType,keepRate);
  fprintf(fidT(funct),'\nProblem Best Median Mean Std Worst\n');
  
  FileNameE{funct}=[FileName{funct} 'Exper.txt'];
  fidE(funct) = fopen(FileNameE{funct}, 'w');
  fprintf(fidE(funct),' restartRate');
  fprintf(fidE(funct),' expAlpha strategies popSizeType');
  fprintf(fidE(funct),' keepRate ');
  fprintf(fidE(funct),'\n  %1.2f %1.2f %s %i  %f  \n', ...
    restartRate, expAlpha, stratString,  popSizeType,keepRate);
  fprintf(fidE(funct),'\nDistribution of the best results');
  
  FileNameI{funct}=[FileName{funct} 'Intermed.txt'];
  fidI(funct) = fopen(FileNameI{funct}, 'w');
  fprintf(fidI(funct),' restartRate');
  fprintf(fidI(funct),' expAlpha strategies popSizeType');
  fprintf(fidI(funct),' keepRate');
  fprintf(fidI(funct),'\n  %1.2f %1.2f %s %i %i %f \n', ...
    restartRate, expAlpha, stratString, popSizeType,keepRate);
  
end

fName=[results, 'Recap.txt'];
fidRecap=fopen(fName,'w');
fprintf(fidRecap,'Function Best Median Mean Std_dev Worst \n');

%-------------------------------------------------Defines the repertories to use
switch funSet
  case 1
    CEC2011Dir = 'CEC 2011 Benchmarks';
    addpath(['./', CEC2011Dir]);
    addpath(['./', CEC2011Dir, '/Probs_1_to_8']);
    addpath(['./', CEC2011Dir, '/Prob_9_Transmission_Pricing']);
    addpath(['./', CEC2011Dir, '/Prob_10_Circ_Antenna']);
    addpath(['./', CEC2011Dir, '/Probs_11_ELD_Package/DED Codes']);
    addpath(['./', CEC2011Dir, '/Probs_11_ELD_Package/ELD Codes']);
    addpath(['./', CEC2011Dir, '/Probs_11_ELD_Package/Hydrothermal Codes']);
    addpath(['./', CEC2011Dir, '/Probs_12_to_13_Package']);
    GetFunInfoAddr = @CEC2011get_fun_info;
    FuncAddr = @CEC2011func;
    fprintf('\nCEC 2011 Real world problems');
    
  case 2
    addpath('./CEC 2005 Benchmarks');
    GetFunInfoAddr = @CEC2005get_fun_info;
    FuncAddr = @CEC2005func;
    fprintf('\nCEC 2005 Problems');
    
  case 3
% Other functions: 
%		Alpine, Tripod, Gear Train, Pressure vessel, Lennard-Jones, etc.
%			See /Myfunctions/get_fun_info.m	for details					
    addpath('./MyFunctions');
    GetFunInfoAddr = @get_fun_info;
    FuncAddr = @func;
    fprintf('\nMy functions');
end
%===============================================================================
nbFunc=length(funArray);
coeff=2450;	%  For "Limited budget" approach
%--------------------------------------------------------- Loop on functions-- 1
for funct = 1:nbFunc
  fun=funArray(funct);
  
  switch funSet
    case 2 % CEC2005
      D=Darray(funct);
      [LB, UB, opt_f, ~] = GetFunInfoAddr(fun,D);
      quantis=[];

     case 3
     if exist('Darray','var') 
    		D0=Darray(funct);
    	else D0=[];
    	 end
    	 
      [LB, UB, D, opt_f, ~,quantis] = GetFunInfoAddr(fun,D0);
      
    otherwise
      [LB, UB, D, opt_f, ~] = GetFunInfoAddr(fun);
      quantis=[];
  end % switch funSet
  
  % Normalises quantis
  if ~isempty(quantis)
  	quantis=quantis./(UB-LB);
  end
  
  fprintf('\nProblem %i, dimension %i',fun,D);
  
  if exist('budgetArray', 'var') && ~isempty(budgetArray)
    FEmax=budgetArray(funct);
  else
    FEmax=100*floor(coeff*sqrt(D)/100);
  end
  
  FEintermed=floor(FEmax*(1:30)/30); % Intermediate FE values for which
  %																	results will be saved on a file
  nbIntermed=length(FEintermed);
  
  for interm=1:nbIntermed
    fprintf(fidI(funct),'%i ',FEintermed(interm));
  end
  
  rng(123456789); % Initialise the RNG for reproducible results
  %  RandSeed = round(sum(clock)); % For unreproducible results
  %             (except just statistically)
  
  % Warm up the RNG
  for t=1:10000	rand;	end
  
  % Defines the population size
  D=length(LB);
  N=popSize(popSizeType,D);
  
  fprintf('\nPopulation size %i',N);
  
  % Saves search space
  fprintf(fid(funct),'\n');
  for d=1:D
    fprintf(fid(funct),'%1.4e ',LB(d));
  end
  fprintf(fid(funct),'\n');
  for d=1:D
    fprintf(fid(funct),'%1.4e ',UB(d));
  end
  
  % Defines trios. Each trio is sorted, and the set is also sorted
  % As later we will select at most 2N ones, there is surely a clever way
  % to do that! On the other hand, it is done just once.
  triosAll=nchoosek(1:N,3);
  
  % Global values for the normalised search space
  diagonal=sqrt(D);
  zerosD=zeros(1,D);
  onesD=ones(1,D);
  xBest=zeros(nbExperiment,D);
 

  %---------------------------------------------------------- Experiments -- 1.1
  fExper=Inf*(1:nbExperiment);
  for nExper=1:nbExperiment
    fprintf('\n--------------------- Experiment %i/%i',nExper,nbExperiment);
    
    fprintf('\nfunction %i, dimension %i, FEmax %i',fun, D, FEmax);
    
    [xBest(nExper,1:D), fExper(nExper),FEs, intermed]= ...
      APS11(funSet, fun, FEmax, opt_f,FuncAddr,LB,UB, expAlpha,contractGamma, ...
      restartRate,strategArray, N,verbose,FEintermed, keepRate, triosAll,...
      quantis);
    
    intermedNb=length(intermed);
    % Complete the intermediate values
    for i=2:intermedNb
      if intermed(i)==Inf intermed(i)=intermed(i-1); end
    end
    intermedMin(1:intermedNb,nExper,funct)=intermed;
    
    % Displays current best result
    fprintf('\n								Min min %1.6e',min(fExper));
    
    % Saves detailed results (best value, best position)
    fprintf(fid(funct),'\n%i %i %1.6e ',nExper, FEs, fExper(nExper));
    fprintf(fid(funct),'\n');
    for d=1:D
      fprintf(fid(funct),'%1.6e ',xBest(nExper,d));
    end
     
  end % for nExper=1:nbExperiment --------------------- End of experiments (1.1)

  
   if signature  
  % Plots signature (normally used only for function Plateau (signature) 
  	figure
		plot(xTot(:,1),xTot(:,2),'LineStyle','none','Marker','o','MarkerFaceColor', 'blue')
		[N,~]=size(xTot);
		fprintf('\n nb of points %i', N); 
	end 

  % -------------------------------------------------------------- Saves results
  
  % Saves a table of best results, for later use (for example in a spreadsheet)
  fprintf(fidE(funct),'\nFunction_%i',fun);
  for nExper=1:nbExperiment
    fprintf(fidE(funct),'\n%1.6e',fExper(nExper));
  end
  
  %  Best over the experiments
  [fBestExper(funct),Ind]=min(fExper);
  fMedianExper(funct)=median(fExper);
  fMeanExper(funct)=mean(fExper);
  fStdExper(funct)= std(fExper);
  fWorstExper(funct)=max(fExper);
  fprintf(fidT(funct),'\n%i %1.6e %1.6e %1.6e %1.6e %1.6e ',fun,...
    fBestExper(funct), fMedianExper(funct), fMeanExper(funct),...
    fStdExper(funct), fWorstExper(funct));
  
  fprintf(fidRecap,'\n%i %1.6e %1.6e %1.6e %1.6e %1.6e ',fun,...
    fBestExper(funct), fMedianExper(funct), fMeanExper(funct),...
    fStdExper(funct), fWorstExper(funct));
  
  
  % Saves the best result
  fprintf(fid(funct),'\nBest position\n');
  for d=1:D
    fprintf(fid(funct),'%1.20e ',xBest(Ind,d));
  end
   
  % Displays the best position
  fprintf('\nBest position:\n')
  disp(xBest(Ind,:));
  
  if plotDistrib==1
  %{
     Plots the distribution of the best results
     and computes some informations about its shape.
     Useful only if the number of experiments is big enough (typically >= 50)
     Note that the data are anyway in the file Exper.txt
    
    	With some strategies that makes an intensive use
    	of randomness, the results are "stabilised"
    	only after at least 1000 experiments
    	So, BE VERY CAREFUL WHEN COMPARING RESULTS
 %}   
    fprintf('\n FEmax %i',FEmax);
    fprintf('\nOver experiments: min         median       mean      std. dev.');
    fprintf('\n                  %1.4e  %1.4e  %1.4e %1.4e',...
      fBestExper(funct), fMedianExper(funct), fMeanExper(funct),fStdExper(funct));
    
    %	Plots the distributions of the best results (density and cumulated)
    mini=fBestExper(funct); maxi= fWorstExper(funct);
    CDFplot (fExper,mini,maxi,nbBins);
  end %  if plotDistrib
  
  % Saves intermediate results (averaged over the number of experiments)
  fprintf(fidI(funct),'\n Intermediate Min \n');
  for interm=1:intermedNb
    u=min(intermedMin(interm,:,funct));
    fprintf(fidI(funct),'%1.6e ',u);
  end
  
  fprintf(fidI(funct),'\n Intermediate Median \n');
  for interm=1:intermedNb
    u=median(intermedMin(interm,:,funct));
    fprintf(fidI(funct),'%1.6e ',u);
  end
  
  
     % Saves all positions for TEST purpose (entropy, in particular)
  if savePos
  	FileNameAllPos{funct}=[FileName{funct} 'All pos.txt'];
  	fidAllPos(funct) = fopen(FileNameAllPos{funct}, 'w');
  	[nbPos, D]=size(xTot);

  	for n=1:nbPos
  		for d=1:D
  			fprintf(fidAllPos,'%1.10e ', xTot(n,d));
  		end
  		fprintf(fidAllPos,'%1.10e \n',fTot(n));
  	end
  end
  
  if funSet==2 % Clear some persistent variables used in CEC 2005 benchmark code
  							% (see benchmark_func)
  	clear fhd f_bias
  	clear o
  	clear M
  	clear a b A alpha
  	clear fun_num func sigma lamda bias 
  end
  
end %for funct = 1:nbFunc  --------------------------End of loop on functions (1)
end
%=============================================================== Population size
function N=popSize(popSizeType,D)
if popSizeType<0  % User-defined size
  N=-popSizeType;
else	% Formulae, depending on the dimension
  switch popSizeType
    case 0
      N=floor(90*(1+4*log(D)/1000));
      
    otherwise
      N=90;if(D>13) N=91; end
  end % switch popSizeType
end % if popSizeType<0
end
